﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using FarPoint.CalcEngine;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Model;

namespace RemovingCustomFunctions_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            fpSpread1.AllowUserFormulas = true;
            fpSpread1.ActiveSheet.ColumnHeader.Columns[0].Label = "Built In Functions";
            fpSpread1.ActiveSheet.ColumnHeader.Columns[1].Label = "Built In Functions";
            fpSpread1.ActiveSheet.ColumnHeader.Columns[3].Label = "Custom Functions";
            fpSpread1.ActiveSheet.ColumnHeader.Columns[4].Label = "Custom Functions";
            fpSpread1.ActiveSheet.Columns[0].Width = 100;
            fpSpread1.ActiveSheet.Columns[1].Width = 100;
            fpSpread1.ActiveSheet.Columns[3].Width = 100;
            fpSpread1.ActiveSheet.Columns[4].Width = 100;
        }

        //*************************** Apply Built-In Functions ***************************
        private void BtnApplyBuiltInFunctions_Click(object sender, EventArgs e)
        {
            fpSpread1.ActiveSheet.Cells[2, 0].Formula = "PRODUCT(A1,5)";
            fpSpread1.ActiveSheet.Cells[1, 1].Formula = "SUM(A1:A4)";
        }

        //*************************** Apply Custom Functions ***************************
        private void btnApplyCustomFunctions_Click(object sender, EventArgs e)
        {
            fpSpread1.ActiveSheet.AddCustomFunction(new CubeFunctionInfo());
            fpSpread1.ActiveSheet.SetFormula(0, 3, "CUBE(5)");

            fpSpread1.ActiveSheet.AddCustomFunction(new XorFunctionInfo());
            fpSpread1.ActiveSheet.SetFormula(1, 4, "XOR(FALSE,TRUE)");
            fpSpread1.ActiveSheet.SetFormula(1, 3, "XOR(TRUE,TRUE)");
        }
        //*************************** Remove Custom Functions ***************************
        private void btnRemoveCustomFunctions_Click(object sender, EventArgs e)
        {
            SheetView sheet = fpSpread1.ActiveSheet;
            DefaultSheetDataModel.DefaultSheetDataModelEnumerator enumerator = ((DefaultSheetDataModel)sheet.Models.Data).EnumFormulas;
            for (int r = enumerator.NextNonEmptyRow(-1); r != -1; r = enumerator.NextNonEmptyRow(r))
            {
                for (int c = enumerator.NextNonEmptyColumnInRow(r, -1); c != -1; c = enumerator.NextNonEmptyColumnInRow(r, c))
                {
                    Expression expr = sheet.GetExpression(r, c);
                    if (ContainsCustomFunction(expr))
                    {
                        sheet.Cells[r, c].Formula = "";
                    }
                }
            }
        }

        //*************** Method to check if function a Custom Function or not ****************
        bool ContainsCustomFunction(Expression expr)
        {
            if (expr is FunctionExpression)
            {
                FunctionExpression funcExpr = (FunctionExpression)expr;
                if (funcExpr.Function == null)
                {
                    return true;
                }
                for (int i = 0; i < funcExpr.ArgCount; i++)
                {
                    if (ContainsCustomFunction(funcExpr.GetArg(i)))
                    {
                        return true;
                    }
                }
            }
            else if (expr is BinaryOperatorExpression)
            {
                BinaryOperatorExpression operExpr = (BinaryOperatorExpression)expr;
                if (ContainsCustomFunction(operExpr.Arg0))
                {
                    return true;
                }
                if (ContainsCustomFunction(operExpr.Arg1))
                {
                    return true;
                }
            }
            else if (expr is UnaryOperatorExpression)
            {
                UnaryOperatorExpression operExpr = (UnaryOperatorExpression)expr;
                if (ContainsCustomFunction(operExpr.Arg))
                {
                    return true;
                }
            }
            else if (expr is ParenthesesExpression)
            {
                ParenthesesExpression parenExpr = (ParenthesesExpression)expr;
                if (ContainsCustomFunction(parenExpr.Arg))
                {
                    return true;
                }
            }
            return false;
        }
    }

    //********************* Creating Custom Functions ***************************
    public class CubeFunctionInfo : FunctionInfo
    {
        public override string Name { get { return "CUBE"; } }
        public override int MinArgs { get { return 1; } }
        public override int MaxArgs { get { return 1; } }
        public override object Evaluate(object[] args)
        {
            double num = CalcConvert.ToDouble(args[0]);
            return num * num * num;
        }
    }
    public class XorFunctionInfo : FunctionInfo
    {
        public override string Name { get { return "XOR"; } }
        public override int MinArgs { get { return 2; } }
        public override int MaxArgs { get { return 2; } }
        public override object Evaluate(object[] args)
        {
            bool arg0 = CalcConvert.ToBool(args[0]);
            bool arg1 = CalcConvert.ToBool(args[1]);
            return (arg0 || arg1) && (arg0 != arg1);
        }
    }
}
